# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

# This is your preferences dialog.
#
# Define your preferences in
# data/glib-2.0/schemas/net.launchpad.diabetes-tracker.gschema.xml
# See http://developer.gnome.org/gio/stable/GSettings.html for more info.

from gi.repository import Gio  # pylint: disable=E0611

from locale import gettext as _

import logging
logger = logging.getLogger('diabetes_tracker')

from diabetes_tracker_lib.PreferencesDialog import PreferencesDialog

# mmol/l * 18.0182 = mg/dl for blood glucose
CONVERSION_FACTOR = 18.0182


class PreferencesDiabetesTrackerDialog(PreferencesDialog):
    __gtype_name__ = "PreferencesDiabetesTrackerDialog"

    def finish_initializing(self, builder):  # pylint: disable=E1002
        """Set up the preferences dialog"""
        super(PreferencesDiabetesTrackerDialog, self).finish_initializing(builder)

        # Bind each preference widget to gsettings
        self.settings = Gio.Settings("net.launchpad.diabetes-tracker")

        self.conversion_factor = 1

        # Code for other initialization actions should be added here.
        self.button1 = builder.get_object('radiobutton1')
        self.button2 = builder.get_object('radiobutton2')
        if self.settings.get_boolean("units") is True:
            self.button1.set_active(True)
        else:
            self.button2.set_active(True)
        self.button1.connect('toggled', self.on_units_toggled, 'radiobutton1')

        self.spinbutton1 = self.builder.get_object("spinbutton1")
        self.spinbutton2 = self.builder.get_object("spinbutton2")
        self.label4 = self.builder.get_object("label4")
        self.label5 = self.builder.get_object("label5")
        self.set_range_preference()

    def set_range_preference(self):
        # True means use mmol/l, False means mg/dl
        if self.settings.get_boolean("units") is True:
            digits = 1
            increments = 0.1
            units = "mmol/l"
            self.conversion_factor = 1
        else:
            digits = 0
            increments = 1
            units = "mg/dl"
            self.conversion_factor = CONVERSION_FACTOR
        self.spinbutton1.set_digits(digits)
        self.spinbutton1.set_increments(increments, 1)
        self.spinbutton1.set_value(round(self.settings.get_double("target-range-low") * self.conversion_factor, 1))
        self.spinbutton2.set_digits(digits)
        self.spinbutton2.set_increments(increments, 1)
        self.spinbutton2.set_value(round(self.settings.get_double("target-range-high") * self.conversion_factor, 1))
        self.label4.set_text(units + " and")
        self.label5.set_text(units)

    def on_low_value_changed(self, value):
        self.settings.set_double("target-range-low", value.get_value() / self.conversion_factor)

    def on_high_value_changed(self, value):
        self.settings.set_double("target-range-high", value.get_value() / self.conversion_factor)

    def on_units_toggled(self, a, b):
        if self.button1.get_active() is True:
            self.settings.set_boolean("units", True)
        else:
            self.settings.set_boolean("units", False)
        self.set_range_preference()
